# coding=utf-8

"""

   Copyright 2015 Andreas Würl

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

class Artifact(object):
    def __init__(self, group_id, artifact_id, package_type, version):
        self.group_id = group_id
        self.artifact_id = artifact_id
        self.package_type = package_type
        self.version = version

    def get_group_id(self):
        return self.group_id

    def get_artifact_id(self):
        return self.artifact_id

    def get_package_type(self):
        return self.package_type

    def get_version(self):
        return self.version

    def __repr__(self):
        return "Artifact[{}:{}:{}:{}]".format(self.group_id, self.artifact_id, self.package_type, self.version)


class Dependency(Artifact):
    def __init__(self, group_id, artifact_id, package_type, version, scope):
        super(Dependency, self).__init__(group_id, artifact_id, package_type, version)
        self.scope = scope

    def get_scope(self):
        return self.scope

    def __repr__(self):
        return "Dependency[{}:{}:{}:{}, scope={}]".format(self.group_id, self.artifact_id, self.package_type,
                                                          self.version, self.scope)



# coding=utf-8

"""

   Copyright 2015 Andreas Würl

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""


class TestReport(object):
    def __init__(self, name, suites_by_module, build_number, is_incremental):
        self.name = name

        self.passed = 0
        self.failures = 0
        self.skipped = 0
        self.errors = 0
        self.duration = 0.0

        for suites in suites_by_module.values():
            self.passed = reduce(lambda value, current_suite: current_suite.passed + value, suites, self.passed)
            self.skipped = reduce(lambda value, current_suite: current_suite.skipped + value, suites, self.skipped)
            self.failures = reduce(lambda value, current_suite: current_suite.failures + value, suites, self.failures)
            self.errors = reduce(lambda value, current_suite: current_suite.errors + value, suites, self.errors)
            self.duration = reduce(lambda value, current_suite: current_suite.duration + value, suites, self.duration)

        self.suites_by_module = suites_by_module
        self.build_number = build_number
        self.is_incremental = is_incremental

    @property
    def total(self):
        return self.passed + self.skipped + self.failures + self.errors

    @property
    def is_successful(self):
        return self.errors == 0 and self.failures == 0 and self.passed > 0

    def __repr__(self):
        return "TestReport(total=" + str(self.total) + ", skipped=" + str(self.skipped) + ", failures=" \
               + str(self.failures) + ", errors=" + str(self.errors) + ")"


class TestSuite(object):
    def __init__(self, name, passed, skipped, failures, errors, duration, testcases, properties=None):
        self.name = name
        self.passed = int(passed)
        self.skipped = int(skipped)
        self.failures = int(failures)
        self.errors = int(errors)
        self.duration = float(duration)
        self.testcases = testcases
        self.properties = properties

    @property
    def total(self):
        return self.passed + self.skipped + self.failures + self.errors


class TestCase(object):
    def __init__(self, classname, name, duration, status):
        self.name = name
        self.classname = classname
        self.duration = float(duration)
        self.status = status


class Failure(object):
    def __init__(self, message, type):
        self.message = message
        self.type = type



# coding=utf-8

"""

   Copyright 2015 Andreas Würl

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

from __future__ import print_function

from subprocess import Popen, PIPE
import re

from .data import Dependency


class DependencyList(object):
    regexp = re.compile(r'([\w\.-]*):([\w\.-]*):(\w*):([\w\.-]*):(\w*)')

    def get_dependencies(self, project_name):
        pipe = Popen(['mvn', 'dependencies:list', '-pl', project_name], stdout=PIPE)
        (dependencies_output, _) = pipe.communicate()

        return self.parse_dependencies(dependencies_output)

    def parse_dependencies(self, output):
        dependency_matches = self.regexp.findall(output)

        return [Dependency(*match) for match in dependency_matches]



import json
import urlparse
import requests
from ..data import TestSuite, TestCase, TestReport


class Report(object):
    CASE_MAP = {'skipped': None, 'failedSince': None, 'className': 'classname', 'age': None}
    SUITE_MAP = {'timestamp': None, 'cases': 'testcases', 'id': None}

    API_JSON = "/api/json"

    def get_report(self, job_url, build_number=None):
        job_url = self.fix_url(job_url)
        project_info = json.loads(requests.get(job_url + self.API_JSON).text)

        if build_number is None:
            build_number = project_info['lastCompletedBuild']['number']
        else:
            build_number = int(build_number)

        build_url = next(
            iter([build['url'] for build in project_info['builds'] if build['number'] == build_number]), None)

        if build_url is None:
            raise ValueError("no build with number {} found".format(build_number))

        build_info = json.loads(requests.get(build_url + self.API_JSON).text)

        test_report_request = requests.get(build_url + '/testReport' + self.API_JSON)

        if test_report_request.status_code != 200:
            return None

        test_report = json.loads(test_report_request.text)

        suites_by_module = {}

        for child_report in test_report['childReports']:
            child_result = child_report['result']
            module_url = child_report['child']['url']
            module = module_url[len(job_url):].split('/')[1].replace('$', ':')

            suites = [self.create_suite(suite) for suite in child_result['suites']]
            print(module_url)
            print(job_url)
            print(module + " with " + str(len(suites)) + " suites")
            suites_by_module[module] = suites

        is_incremental = bool([cause for cause in self.get_causes(build_info['actions'])
                               if cause['shortDescription'] == "Started by an SCM change"])

        print("causes: " + ", ".join(
            [cause['shortDescription'] for cause in self.get_causes(build_info['actions'])]))

        return TestReport(build_info['displayName'], suites_by_module,
                          build_number, is_incremental)

    def create_suite(self, suite_properties):
        suite_properties = self.map_keys(suite_properties, self.SUITE_MAP)

        testcases = [self.create_case(case) for case in suite_properties['testcases']]
        suite_properties['testcases'] = testcases
        suite_properties['passed'] = len([testcase for testcase in testcases if testcase.status == 'PASSED'])
        suite_properties['skipped'] = len([testcase for testcase in testcases if testcase.status == 'SKIPPED'])
        suite_properties['failures'] = len([testcase for testcase in testcases if testcase.status == 'FAILED'])
        suite_properties['errors'] = len([testcase for testcase in testcases if testcase.status == 'ERROR'])

        suite = TestSuite(**suite_properties)
        print("  " + str(suite))
        return suite

    def create_case(self, case_properties):
        case_properties = self.map_keys(case_properties, self.CASE_MAP)
        return TestCase(**case_properties)

    def map_keys(self, target, key_map):
        for old_key, new_key in key_map.iteritems():
            value = target.pop(old_key)
            if value is not None and new_key is not None:
                target[new_key] = value

        return target

    def get_causes(self, actions):
        for action in actions:
            if 'causes' in action:
                return action['causes']
        return []

    def fix_url(self, url):
        return urlparse.urlunparse(urlparse.urlparse(url))



# coding=utf-8

"""

   Copyright 2015 Andreas Würl

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

from __future__ import print_function
import os

import unittest
from assertpy import assert_that

from citools.maven.dependencies import DependencyList


class TestDependencies(unittest.TestCase):
    def setUp(self):
        self.dependency_list = DependencyList()

    def test_parsing_of_dependency_list_output(self):
        with open(os.path.join(os.path.dirname(__file__), 'mvn_dependency_list_output.txt'), 'r') as output_file:
            output = output_file.read()

        dependencies = self.dependency_list.parse_dependencies(output)

        assert_that(dependencies).is_length(22)
